@@ -2,27 +2,27 @@
  *
  * Decode ssh2 keys.
  *
- * $Id: lsh-decode-key.c,v 1.16 2002/01/14 01:16:41 nisse Exp $
  */
 
-#include "digits.h"
-#include "dsa.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <fcntl.h>
+#include <unistd.h>
+
+#include "nettle/sexp.h"
+
 #include "format.h"
 #include "io.h"
 #include "lsh_argp.h"
+#include "lsh_string.h"
 #include "publickey_crypto.h"
-#include "read_file.h"
-#include "sexp.h"
 #include "spki.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <string.h>
-
-#include <unistd.h>
-#include <fcntl.h>
-
 #include "lsh-decode-key.c.x"
 
 /* Option parsing */
@@ -40,8 +40,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        (file string)
 
        ; Assume input is base64
-       (base64 . int)
-       (style . sexp_argp_state)))
+       (base64 . int)))
 */
 
 static struct lsh_decode_key_options *
@@ -50,7 +49,6 @@ make_lsh_decode_key_options(void)
   NEW(lsh_decode_key_options, self);
   self->file = NULL;
   self->base64 = 0;
-  self->style = -1;
 
   return self;
 }
@@ -67,7 +65,6 @@ main_options[] =
 static const struct argp_child
 main_argp_children[] =
 {
-  { &sexp_output_argp, 0, NULL, 0 },
   { &werror_argp, 0, "", 0 },
   { NULL, 0, NULL, 0}
 };
@@ -82,13 +79,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
     default:
       return ARGP_ERR_UNKNOWN;
 
-    case ARGP_KEY_INIT:
-      state->child_inputs[0] = &self->style;
-      break;
-
     case ARGP_KEY_END:
-      if (self->style < 0)
-	self->style = self->file ? SEXP_CANONICAL : SEXP_TRANSPORT;
       break;
       
     case 'b':
@@ -113,13 +104,13 @@ main_argp =
 };
 
 
-static struct sexp *
+static struct lsh_string *
 lsh_decode_key(struct lsh_string *contents)
 {
   struct simple_buffer buffer;
   int type;
 
-  simple_buffer_init(&buffer, contents->length, contents->data);
+  simple_buffer_init(&buffer, STRING_LD(contents));
 
   if (!parse_atom(&buffer, &type))
     {
@@ -133,7 +124,7 @@ lsh_decode_key(struct lsh_string *contents)
       {
         struct verifier *v;
         
-        werror("lsh-decode-key: Reading key of type ssh-dss...\n");
+        werror("Reading key of type ssh-dss...\n");
 
         v = parse_ssh_dss_public(&buffer);
         
@@ -143,14 +134,14 @@ lsh_decode_key(struct lsh_string *contents)
             return NULL;
           }
         else
-          return spki_make_public_key(v);
+          return PUBLIC_SPKI_KEY(v, 1);
       }
       
     case ATOM_SSH_RSA:
       {
           struct verifier *v;
           
-          werror("lsh-decode-key: Reading key of type ssh-rsa...\n");
+          werror("Reading key of type ssh-rsa...\n");
 
           v = parse_ssh_rsa_public(&buffer);
 
@@ -160,7 +151,7 @@ lsh_decode_key(struct lsh_string *contents)
               return NULL;
             }
           else
-            return spki_make_public_key(v);
+            return PUBLIC_SPKI_KEY(v, 1);
       }      
     default:
       werror("Unknown key type.");
@@ -175,7 +166,6 @@ int main(int argc, char **argv)
   const struct exception *e;
   struct lsh_string *input;
   struct lsh_string *output;
-  struct sexp *expr;
   
   int out = STDOUT_FILENO;
   
@@ -187,8 +177,8 @@ int main(int argc, char **argv)
                  O_WRONLY | O_CREAT, 0666);
       if (out < 0)
         {
-          werror("Failed to open file `%S' for writing: %z\n",
-                 options->file, STRERROR(errno));
+          werror("Failed to open file `%S' for writing %e\n",
+                 options->file, errno);
           return EXIT_FAILURE;
         }
     }
@@ -196,34 +186,32 @@ int main(int argc, char **argv)
   input = io_read_file_raw(STDIN_FILENO, 3000);
   if (!input)
     {
-      werror("Failed to read stdin: %z\n",
-             STRERROR(errno));
+      werror("Failed to read stdin %e\n", errno);
       return EXIT_FAILURE;
     }
 
   if (options->base64)
     {
-      struct lsh_string *old = input;
-      input = decode_base64(input->length, input->data);
-      lsh_string_free(old);
-
-      if (!input)
-        {
+      if (!lsh_string_base64_decode(input))
+	{
           werror("Invalid base64 encoding.\n");
+
+	  lsh_string_free(input);
+
           return EXIT_FAILURE;
         }
     }
-
-  expr = lsh_decode_key(input);
-
-  lsh_string_free(input);
   
-  if (!expr)
-    return EXIT_FAILURE;
+  output = lsh_decode_key(input);
+  lsh_string_free(input);
 
-  output = sexp_format(expr, options->style, 0);
-
-  e = write_raw(out, output->length, output->data);
+  if (!output)
+    {
+      werror("Invalid ssh2 key.\n");
+      return EXIT_FAILURE;
+    }
+    
+  e = write_raw(out, STRING_LD(output));
   lsh_string_free(output);
   
   if (e)
